/*-----------------------------------V2.1--------------------------------------------------------------------
 * Décodeur DCC pour servomoteurs (jusqu'à 8) pour arduino nano
 * l'entrée:
 *   le signal DCC en D2 (interruption 0)
 * les sorties: (ne se suit pas pour simplification des straps)
 *  Servo1 en D3
 *  Servo2 en D5
 *  Servo3 en D7
 *  Servo4 en D4
 *  Servo5 en D6
 *  Servo6 en D8
 *  Servo7 en D10
 *  Servo8 en D12
 *  Led de controle en D9
 *  Leds amplifiées par un ULN2003 (7Leds) ou un ULN2803(8Leds)
 *  Led 1 en A0
 *  Led 2 en A1
 *  Led 3 en A2
 *  Led 4 en A3
 *  Led 5 en A4
 *  Led 6 en A5
 *  Led 7 en D13
 *  Led 8 en D11 (avec un ULN2803)
 *  -------------------------------------------------------------------------------------------------------
 * Je travaille qu'avec le "Chiffre Unique"
 * A partir du n° de l'accessoire ECOS
 *  CU = ECOS*2+6
 * à partir de l'interruption 
 *  CU = (address*8)+data
 *  CU = ON, CU+1 = OFF
 *  --------------------------------------------------------------------------------------------------------
 *  Il suffit de mettre le numéro de l'accessoire ECOS après servoX
 *  pour que le compilateur ne prenne en compte QUE les servos initialisés
 *  le servoX à ZERO ou absent n'est pas pris en compte par le compilateur 
 *  mettre le chiffre en négatif pour inverser le sens du servo
 *  Tpx est le temps d'un cycle d'allumage et d'extinction
 *  Rapx est le temps d'allumage dans le cycle quand n°accessoire est positif
 *  Rapx est le temps d'extinction dans le cycle quand n°accessoire est négatif
----------------------------------------------------------------------------------------------------------*/
//   ICI ON DOIT PARAMETRER LE DECODEUR
//   servo1=D3 servo2=D5 servo3=D7 servo4=D4 servo5=D6 servo7=D10 servo8=D12
//   led1=A0 led2=A1 led3=A2 led4=A3 led5=A4 led6=A5 led7=D13 led8=D11
//   D9= led de contrôle 
//----------------------------------------------------------------------------------------------------
#define servo1 600      // n°de l'accessoire ici le 600 (droite vers gauche)
#define servo2 -601     //si négatif cela inverse le mouvement du servo ici le 601 (gauche vers droite)
#define servo3 602
#define servo4 603
#define servo5 604
#define servo6 605
#define servo7 0        // non déclaré.....aucun code sera généré
#define servo8 0

#define vitesse 10        // vitesse des servos en milliseconde (25 lent à 5 rapide)

#define led1 606     //N° de l'accessoire Si positif le cycle est  allumage puis extinction
#define Tps1 4      // Nombre de 1/10 seconde d'un cycle (allumage + extinction) ou (extinction + allumage)
#define Rap1 4      // Nombre de 1/10 seconde d'allumage (si Tps1=Rapx pas de clignotement (on/off))
//-------------------
#define led2 607    //N° de l'accessoire Si négatif cela inverse le cycle (extinction puis allumage)
#define Tps2 8      // temps du cycle 0.8 seconde
#define Rap2 2      // Allumé pendant 0.2 secondes éteint pendant 0.6 secondes (8-2)
//-------------------
#define led3 -608   // Inverse le cycle (eteint- allumé)
#define Tps3 10     // Nombre de 1/10 seconde d'un cycle (allumage + extinction)
#define Rap3 5      // si Tps1=Rapx pas de clignotement (on/off) et la valeur du chiffre n'est pas importante
//-------------------
#define led4 609    // avec la suivante (LED5) cela fera deux led qui clignoteront en alternées
#define Tps4 6      
#define Rap4 3      
//-------------------
#define led5 -609
#define Tps5 6      
#define Rap5 3      
//-------------------
#define led6 611
#define Tps6 6      // Nombre de 1/10 seconde d'un cycle (allumage + extinction)
#define Rap6 6      // Nombre de 1/10 seconde d'allumage (si Tps1=Rapx pas de clignotement (on/off))
//-------------------
#define led7 612
#define Tps7 6      // Nombre de 1/10 seconde d'un cycle (allumage + extinction)
#define Rap7 6      // Nombre de 1/10 seconde d'allumage (si Tps1=Rapx pas de clignotement (on/off))
//-------------------
#define led8 613
#define Tps8 6      // Nombre de 1/10 seconde d'un cycle (allumage + extinction)
#define Rap8 6      // Nombre de 1/10 seconde d'allumage (si Tps1=Rapx pas de clignotement (on/off))
//-------------------

#define test          // si test est déclaré on affiche le "sérial" à mettre en remarque pour le programme final
//----------------------------------------------------------------------------
//                 A PARTIR D'ICI ON NE TOUCHE A RIEN
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Déclaration des librairies untilisées
//---------------------------------------------------------------------------
#include <DCC_Decoder.h>              // librairie DCC
#include <Servo.h>                    // librairie pour gérer les servos
#include <MsTimer2.h>                 // librairie pour créer un timer
//---------------------------------------------------------------------------
// Déclaration des servos
//---------------------------------------------------------------------------
struct servoItem {
  int angle;           // position actuelle du servo
  byte borne1;          // borne1 et borne2 sont les angles extrèmes de 0 à 180 degrés
  byte borne2;          // le servo ira de borne1 à borne2 (et visversa) borne2 peut être plus petit que borne1 ce qui gére l'inversion de sens
  byte cible;           // variable de travail 0 on est arrivé 1 vers borne1 2 vers borne2
  Servo servo;
};

//---------------------------------------------------------------------------
// Déclaration des variables de compilateur
//---------------------------------------------------------------------------
#define kDCC_INTERRUPT    0              // les interruptions arrivent en pin 2

//---------------------------------------------------------------------------
// Variables globales du programme
//---------------------------------------------------------------------------
int OldChiffreUnique=0;             // Chiffre unique précédent pour ne pas faire des doubles traitements
unsigned long OldMillis;            // Mémorisation du temps pour la vitesse des servos
int servotimer;                     // vitesse des servos (5-20) modifiable par le #define


//-------------------------- Création des variables pour les servos......s'ils sont déclarés
#if servo1!=0         // je n'utilise pas #ifdef car si =0 il est défini
  servoItem Servo1;   int CU1;
#endif
#if servo2!=0
  servoItem Servo2;   int CU2;
#endif
#if servo3!=0
  servoItem Servo3;  int CU3;
#endif
#if servo4!=0
  servoItem Servo4;  int CU4;
#endif
#if servo5!=0
  servoItem Servo5;  int CU5;
#endif
#if servo6!=0
  servoItem Servo6;  int CU6;
#endif
#if servo7!=0
  servoItem Servo7;  int CU7;
#endif
#if servo8!=0
  servoItem Servo8;  int CU8;
#endif

//----------------------------- Création des variables si LEDX est déclarée
#if led1!=0   
int LU1; byte Active1; byte PosLed1;
#endif
#if led2!=0   
int LU2; byte Active2; byte PosLed2;
#endif
#if led3!=0   
int LU3; byte Active3; byte PosLed3;
#endif
#if led4!=0   
int LU4; byte Active4; byte PosLed4;
#endif
#if led5!=0   
int LU5; byte Active5; byte PosLed5;
#endif
#if led6!=0   
int LU6; byte Active6; byte PosLed6;
#endif
#if led7!=0   
int LU7; byte Active7; byte PosLed7;
#endif
#if led8!=0   
int LU8; byte Active8; byte PosLed8;
#endif

/************************************************************************************* 
 *  Procédure qui s'exécute à chaque message DCC sur le rail
 ***********************************************************************************/
void DCC_CallBack(int address, boolean activate, byte data)
{  int ChiffreUnique = (address*8)+data;
#ifdef test
Serial.print(data);
Serial.print("  ");
Serial.print(activate);
Serial.print("  ");
Serial.println(address);
Serial.println(ChiffreUnique);
#endif
  if(ChiffreUnique==OldChiffreUnique){ return;}
  OldChiffreUnique=ChiffreUnique;
// le compilateur ne prendra que ce qui sera néccessaire
#if servo1>0
    if(ChiffreUnique==CU1){Servo1.cible=1;Servo1.servo.attach(3);}
    if(ChiffreUnique==CU1+1){Servo1.cible=2;Servo1.servo.attach(3);}
#endif
#if servo1<0
    if(ChiffreUnique==CU1){Servo1.cible=2;Servo1.servo.attach(3);}
    if(ChiffreUnique==CU1+1){Servo1.cible=1;Servo1.servo.attach(3);}
#endif

#if servo2>0
    if(ChiffreUnique==CU2){Servo2.cible=1;Servo2.servo.attach(5);}
    if(ChiffreUnique==CU2+1){Servo2.cible=2;Servo2.servo.attach(5);}
#endif
#if servo2<0
    if(ChiffreUnique==CU2){Servo2.cible=2;Servo2.servo.attach(5);}
    if(ChiffreUnique==CU2+1){Servo2.cible=1;Servo2.servo.attach(5);}
#endif

#if servo3>0
    if(ChiffreUnique==CU3){Servo3.cible=1;Servo3.servo.attach(7);}
    if(ChiffreUnique==CU3+1){Servo3.cible=2;Servo3.servo.attach(7);}
#endif
#if servo3<0
    if(ChiffreUnique==CU3){Servo3.cible=2;Servo3.servo.attach(7);}
    if(ChiffreUnique==CU3+1){Servo3.cible=1;Servo3.servo.attach(7);}
#endif

#if servo4>0
    if(ChiffreUnique==CU4){Servo4.cible=1;Servo4.servo.attach(4);}
    if(ChiffreUnique==CU4+1){Servo4.cible=2;Servo4.servo.attach(4);}
#endif
#if servo4<0
    if(ChiffreUnique==CU4){Servo4.cible=2;Servo4.servo.attach(4);}
    if(ChiffreUnique==CU4+1){Servo4.cible=1;Servo4.servo.attach(4);}
#endif

#if servo5>0
    if(ChiffreUnique==CU5){Servo5.cible=1;Servo5.servo.attach(6);}
    if(ChiffreUnique==CU5+1){Servo5.cible=2;Servo5.servo.attach(6);}
#endif
#if servo5<0
    if(ChiffreUnique==CU5){Servo5.cible=2;Servo5.servo.attach(6);}
    if(ChiffreUnique==CU5+1){Servo5.cible=1;Servo5.servo.attach(6);}
#endif

#if servo6>0
    if(ChiffreUnique==CU6){Servo6.cible=1;Servo6.servo.attach(8);}
    if(ChiffreUnique==CU6+1){Servo6.cible=2;Servo6.servo.attach(8);}
#endif
#if servo6<0
    if(ChiffreUnique==CU6){Servo6.cible=2;Servo6.servo.attach(8);}
    if(ChiffreUnique==CU6+1){Servo6.cible=1;Servo6.servo.attach(8);}
#endif

#if servo7>0
    if(ChiffreUnique==CU7){Servo7.cible=1;Servo7.servo.attach(10);}
    if(ChiffreUnique==CU7+1){Servo7.cible=2;Servo7.servo.attach(10);}
#endif
#if servo7<0
    if(ChiffreUnique==CU7){Servo7.cible=2;Servo7.servo.attach(10);}
    if(ChiffreUnique==CU7+1){Servo7.cible=1;Servo7.servo.attach(10);}
#endif

#if servo8>0
    if(ChiffreUnique==CU8){Servo8.cible=1;Servo8.servo.attach(12);}
    if(ChiffreUnique==CU8+1){Servo8.cible=2;Servo8.servo.attach(12);}
#endif
#if servo8<0
    if(ChiffreUnique==CU8){Servo8.cible=2;Servo8.servo.attach(12);}
    if(ChiffreUnique==CU8+1){Servo8.cible=1;Servo8.servo.attach(12);}
#endif

#if led1!=0    //A0
    if(ChiffreUnique==LU1){Active1=0;}
    if(ChiffreUnique==LU1+1){Active1=1;PosLed1=0;}
#endif
#if led2!=0    //A1
    if(ChiffreUnique==LU2){Active2=0;}
    if(ChiffreUnique==LU2+1){Active2=1;PosLed2=0;}
#endif
#if led3!=0    //A2
    if(ChiffreUnique==LU3){Active3=0;}
    if(ChiffreUnique==LU3+1){Active3=1;PosLed3=0;}
#endif
#if led4!=0    //A3
    if(ChiffreUnique==LU4){Active4=0;}
    if(ChiffreUnique==LU4+1){Active4=1;PosLed4=0;}
#endif
#if led5!=0    //A4
    if(ChiffreUnique==LU5){Active5=0;}
    if(ChiffreUnique==LU5+1){Active5=1;PosLed5=0;}
#endif
#if led6!=0    //A5
    if(ChiffreUnique==LU6){Active6=0;}
    if(ChiffreUnique==LU6+1){Active6=1;PosLed6=0;}
#endif
#if led7!=0    //D13
    if(ChiffreUnique==LU7){Active7=0;}
    if(ChiffreUnique==LU7+1){Active7=1;PosLed7=0;}
#endif
#if led8!=0    //D11
    if(ChiffreUnique==LU8){Active8=0;}
    if(ChiffreUnique==LU8+1){Active8=1;PosLed8=0;}
#endif
}
/**********************************************************************************************
 *  Gestion des LED
 ******************************************************************************************/
void GestionLed() {
// -----------------------------A0
#if led1!=0
if(Active1==0) {PORTC &= ~(1<<0);} 
else {
#if led1>0
  if(PosLed1>Rap1) {PORTC &= ~(1<<0);} else {PORTC |=  (1<<0);}
#endif
#if led1<0
  if(PosLed1>Rap1)  {PORTC |=  (1<<0);} else {PORTC &= ~(1<<0);}
#endif
  PosLed1++; if(PosLed1>Tps1)PosLed1=0;
}
#endif
// --------------------------------------------------A1
#if led2!=0
if(Active2==0) {PORTC &= ~(1<<1);}   
else {
#if led2>0
  if(PosLed2>Rap2) {PORTC &= ~(1<<1);} else {PORTC |=  (1<<1);}
#endif
#if led2<0
  if(PosLed2>Rap2)  {PORTC |=  (1<<1);} else {PORTC &= ~(1<<1);}
#endif
  PosLed2++;   if(PosLed2>Tps2)PosLed2=0;
}
#endif
// -----------------------------------------------A2 (led3)
#if led3!=0
if(Active3==0) {PORTC &= ~(1<<2);}   
else {
#if led3>0
  if(PosLed3>Rap3) {PORTC &= ~(1<<2);} else {PORTC |=  (1<<2);}
#endif
#if led3<0
  if(PosLed3>Rap3)  {PORTC |=  (1<<2);} else {PORTC &= ~(1<<2);}
#endif
PosLed3++;   if(PosLed3>Tps3)PosLed3=0;
}
#endif
//------------------------------------------- A3 (led4)
#if led4!=0
if(Active4==0) {PORTC &= ~(1<<3);}   
else {
#if led4>0
  if(PosLed4>Rap4) {PORTC &= ~(1<<3);} else {PORTC |=  (1<<3);}
#endif
#if led4<0
  if(PosLed4>Rap4) {PORTC |=  (1<<3);} else {PORTC &= ~(1<<3);}
#endif
  PosLed4++; if(PosLed4>Tps4)PosLed4=0;
}
#endif
// -------------------------------------------------A4
#if led5!=0
if(Active5==0) {PORTC &= ~(1<<4);}  
else {
#if led5>0
  if(PosLed5>Rap5) {PORTC &= ~(1<<4);} else {PORTC |=  (1<<4);}
#endif
#if led5<0
  if(PosLed5>Rap5) {PORTC |=  (1<<4);} else {PORTC &= ~(1<<4);}
#endif
  PosLed5++;  if(PosLed5>Tps5)PosLed5=0;
}
#endif
// ----------------------------------------------------A5
#if led6!=0
if(Active6==0) {PORTC &= ~(1<<5);}   
else {
#if led6>0
  if(PosLed6>Rap6) {PORTC &= ~(1<<5);} else {PORTC |=  (1<<5);}
#endif
#if led6<0
  if(PosLed6>Rap6) {PORTC |=  (1<<5);} else {PORTC &= ~(1<<5);}
#endif
  PosLed6++;  if(PosLed6>Tps6)PosLed6=0;
}
#endif
// ---------------------------------------------------D13
#if led7!=0
if(Active7==0) {PORTB &= ~(1<<5);}   
else {
#if led7>0
  if(PosLed7>Rap7) {PORTB &= ~(1<<5);} else {PORTB |=  (1<<5);}
#endif
#if led7<0
  if(PosLed7>Rap7) {PORTB |=  (1<<5);} else {PORTB &= ~(1<<5);}
#endif
  PosLed7++;  if(PosLed7>Tps7)PosLed7=0;
}
#endif
// -----------------------------------------------D11  (led8)
#if led8!=0
if(Active8==0) {PORTB &= ~(1<<3);}   
else {
#if led8>0
  if(PosLed8>Rap8) {PORTB &= ~(1<<3);} else {PORTB |=  (1<<3);}
#endif
#if led8<0
  if(PosLed8>Rap8) {PORTB |=  (1<<3);} else {PORTB &= ~(1<<3);}
#endif
  PosLed8++;  if(PosLed8>Tps8)PosLed8=0;
}
#endif
}

/**********************************************************************************************
 *  setup
 ******************************************************************************************/
void setup() {
#ifdef test
Serial.begin(9600); //Initialisation de la console
Serial.println("Initialisation");     //On écrit dans la console
#endif    
DCC.SetBasicAccessoryDecoderPacketHandler(DCC_CallBack, true);   // Déclaration de la procédure de retour DCC
  DCC.SetupDecoder( 0x00, 0x00, kDCC_INTERRUPT );                  // Sur l'interruption "0" donc sur D2
  pinMode(12,OUTPUT);
  MsTimer2::set(100, GestionLed); // toutes les 100ms (1/10 seconde) on excécute 'GestionLed'
  MsTimer2::start();  
  
  servotimer=10;
  #if vitesse>0     // si on a déclaré 'vitesse' on prend sa valeur
    servotimer = vitesse;
  #endif
#ifdef test
  Serial.print("vitesse -> ");
  Serial.println(servotimer);
#endif
  OldMillis = millis();

// Initialisation du servo n°1 s'il a été déclaré
#if servo1!=0
  // --------------on peut modifier borne1 et borne2
  Servo1.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo1.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo1.angle=90;
  Servo1.cible=0;
  CU1=abs(servo1)*2+6;
#ifdef test
  Serial.print("Servo 1 -> ");
  Serial.print(abs(servo1));
  Serial.print(" CU ");
  if(servo1<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU1);
#endif
#endif

// Initialisation du servo n°2 s'il a été déclaré
#if servo2!=0
  // --------------on peut modifier borne1 et borne2
  Servo2.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo2.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo2.cible=0;
  Servo2.angle=90;
  CU2=abs(servo2)*2+6;
#ifdef test
  Serial.print("Servo 2 -> ");
  Serial.print(abs(servo2));
  Serial.print(" CU ");
  if(servo2<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU2);
#endif
#endif

// Initialisation du servo n°3 s'il a été déclaré
#if servo3!=0
  // --------------on peut modifier borne1 et borne2
  Servo3.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo3.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo3.cible=0;
  Servo3.angle=90;
  CU3=abs(servo3)*2+6;
#ifdef test
  Serial.print("Servo 3 -> ");
  Serial.print(abs(servo3));
  Serial.print(" CU ");
  if(servo3<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU3);
#endif
#endif

// Initialisation du servo n°4 s'il a été déclaré
#if servo4!=0
  // --------------on peut modifier borne1 et borne2
  Servo4.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo4.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo4.cible=0;
  Servo4.angle=90;
  CU4=abs(servo4)*2+6;
#ifdef test
  Serial.print("Servo 4 -> ");
  Serial.print(abs(servo4));
  Serial.print(" CU ");
  if(servo4<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU4);
#endif
#endif

// Initialisation du servo n°5 s'il a été déclaré
#if servo5!=0
  // --------------on peut modifier borne1 et borne2
  Servo5.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo5.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo5.cible=0;
  Servo5.angle=90;
  CU5=abs(servo5)*2+6;
#ifdef test
  Serial.print("Servo 5 -> ");
  Serial.print(abs(servo5));
  Serial.print(" CU ");
  if(servo5<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU5);
#endif
#endif

// Initialisation du servo n°6 s'il a été déclaré
#if servo6!=0
  // --------------on peut modifier borne1 et borne2
  Servo6.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo6.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo6.cible=0;
  Servo6.angle=90;
  CU6=abs(servo6)*2+6;
#ifdef test
  Serial.print("Servo 6 -> ");
  Serial.print(abs(servo6));
  Serial.print(" CU ");
  if(servo6<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU6);
#endif
#endif

// Initialisation du servo n°7 s'il a été déclaré
#if servo7!=0
  // --------------on peut modifier borne1 et borne2
  Servo7.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo7.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo7.cible=0;
  Servo7.angle=90;
  CU7=abs(servo7)*2+6;
#ifdef test
  Serial.print("Servo 7 -> ");
  Serial.print(abs(servo7));
  Serial.print(" CU ");
  if(servo7<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU7);
#endif
#endif

// Initialisation du servo n°8 s'il a été déclaré
#if servo8!=0
  // --------------on peut modifier borne1 et borne2
  Servo8.borne1=0;             //valeur minimum et maximun angle entre 0 et 180)
  Servo8.borne2=180;           //valeur minimum et maximun angle entre 0 et 180)
  //----------------------------------------------------------------------------
  Servo8.cible=0;
  Servo8.angle=90;
  CU8=abs(servo8)*2+6;
#ifdef test
  Serial.print("Servo 8 -> ");
  Serial.print(abs(servo8));
  Serial.print(" CU ");
  if(servo8<0) Serial.print(" <- "); else Serial.print(" -> ");
  Serial.println(CU8);
#endif
#endif
//--------------------------------------------------Les LED
#if led1!=0
  pinMode(14,OUTPUT);  digitalWrite(14,LOW);  LU1=abs(led1)*2+6;
#ifdef test
  Serial.print("Led 1 -> ");
  Serial.print(abs(led1));
  if(led1<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU1);
#endif
#endif

#if led2!=0
  pinMode(15,OUTPUT);  digitalWrite(15,LOW);   LU2=abs(led2)*2+6;
#ifdef test
  Serial.print("Led 2 -> ");
  Serial.print(abs(led2));
  if(led2<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU2);
#endif
#endif

#if led3!=0
  pinMode(16,OUTPUT);  digitalWrite(16,LOW);  LU3=abs(led3)*2+6;
#ifdef test
  Serial.print("Led 3 -> ");
  Serial.print(abs(led3));
  if(led3<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU3);
#endif
#endif

#if led4!=0
  pinMode(17,OUTPUT);  digitalWrite(17,LOW);  LU4=abs(led4)*2+6;
#ifdef test
  Serial.print("Led 4 -> ");
  Serial.print(abs(led4));
  if(led4<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU4);
#endif
#endif

#if led5!=0
  pinMode(18,OUTPUT);   digitalWrite(18,LOW);   LU5=abs(led5)*2+6;
#ifdef test
  Serial.print("Led 5 -> ");
  Serial.print(abs(led5));
  if(led5<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU5);
#endif
#endif

#if led6!=0
  pinMode(19,OUTPUT);   digitalWrite(19,LOW);   LU6=abs(led6)*2+6;
#ifdef test
  Serial.print("Led 6 -> ");
  Serial.print(abs(led6));
  if(led6<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU6);
#endif
#endif

#if led7!=0
  pinMode(13,OUTPUT);   digitalWrite(13,LOW);   LU7=abs(led7)*2+6;
#ifdef test
  Serial.print("Led 7 -> ");
  Serial.print(abs(led7));
  if(led7<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU7);
#endif
#endif

#if led8!=0
  pinMode(11,OUTPUT);   digitalWrite(11,LOW);   LU8=abs(led8)*2+6;
#ifdef test
  Serial.print("Led 8 -> ");
  Serial.print(abs(led8));
  if(led8<0) Serial.print(" (E/A) "); else Serial.print(" (A/E) ");
  Serial.print(" CU ");
  Serial.println(LU8);
#endif
#endif


  // Information de fin initialisation
#ifdef test
  Serial.println("Fin d'initialisation");     //On écrit dans la console en mode TEST
#else
  // clignotement de la led pour fin d'initialisation si pas en mode test
  PORTB |=  (1<<1);
  delay(200);
  PORTB &= ~(1<<1) ;
  delay(200);
  PORTB |=  (1<<1);
  delay(200);
  PORTB &= ~(1<<1) ;
#endif
}


/*************************************************************************
 * loop
 ******************************************************************/
void loop() {
  boolean led = false;
  DCC.loop(); 

  if(millis()>OldMillis+servotimer){
    OldMillis=millis();


#if servo1!=0
  if(Servo1.cible==1){
      if(Servo1.angle==Servo1.borne1){
        Servo1.servo.detach();
        Servo1.cible=0;
        } else{    
        Servo1.angle--;
        Servo1.servo.write(Servo1.angle);
        led=true;
      }
  }
  if(Servo1.cible==2){
      if(Servo1.angle==Servo1.borne2){
        Servo1.servo.detach();
        Servo1.cible=0;
        }else{
        Servo1.angle++;
        Servo1.servo.write(Servo1.angle);
        led=true;
      }
  }
#endif

#if servo2!=0
  if(Servo2.cible==1){
      if(Servo2.angle==Servo2.borne1){
        Servo2.servo.detach();
        Servo2.cible=0;
        } else{    
        Servo2.angle--;
        Servo2.servo.write(Servo2.angle);
        led=true;
      }
  }
  if(Servo2.cible==2){
      if(Servo2.angle==Servo2.borne2){
        Servo2.servo.detach();
        Servo2.cible=0;
        }else{
        Servo2.angle++;
        Servo2.servo.write(Servo2.angle);
        led=true;
      }
  }
#endif

#if servo3!=0
  if(Servo3.cible==1){
      if(Servo3.angle==Servo3.borne1){
        Servo3.servo.detach();
        Servo3.cible=0;
        } else{    
        Servo3.angle--;
        Servo3.servo.write(Servo3.angle);
        led=true;
      }
  }
  if(Servo3.cible==2){
      if(Servo3.angle==Servo3.borne2){
        Servo3.servo.detach();
        Servo3.cible=0;
        }else{
        Servo3.angle++;
        Servo3.servo.write(Servo3.angle);
        led=true;
      }
  }
#endif

#if servo4!=0
  if(Servo4.cible==1){
      if(Servo4.angle==Servo4.borne1){
        Servo4.servo.detach();
        Servo4.cible=0;
        } else{    
        Servo4.angle--;
        Servo4.servo.write(Servo4.angle);
        led=true;
      }
  }
  if(Servo4.cible==2){
      if(Servo4.angle==Servo4.borne2){
        Servo4.servo.detach();
        Servo4.cible=0;
        }else{
        Servo4.angle++;
        Servo4.servo.write(Servo4.angle);
        led=true;
      }
  }
#endif

#if servo5!=0
  if(Servo5.cible==1){
      if(Servo5.angle==Servo5.borne1){
        Servo5.servo.detach();
        Servo5.cible=0;
        } else{    
        Servo5.angle--;
        Servo5.servo.write(Servo5.angle);
        led=true;
      }
  }
  if(Servo5.cible==2){
      if(Servo5.angle==Servo5.borne2){
        Servo5.servo.detach();
        Servo5.cible=0;
        }else{
        Servo5.angle++;
        Servo5.servo.write(Servo5.angle);
        led=true;
      }
  }
#endif

#if servo6!=0
  if(Servo6.cible==1){
      if(Servo6.angle==Servo6.borne1){
        Servo6.servo.detach();
        Servo6.cible=0;
        } else{    
        Servo6.angle--;
        Servo6.servo.write(Servo6.angle);
        led=true;
      }
  }
  if(Servo6.cible==2){
      if(Servo6.angle==Servo6.borne2){
        Servo6.servo.detach();
        Servo6.cible=0;
        }else{
        Servo6.angle++;
        Servo6.servo.write(Servo6.angle);
        led=true;
      }
  }
#endif

#if servo7!=0
  if(Servo7.cible==1){
      if(Servo7.angle==Servo7.borne1){
        Servo7.servo.detach();
        Servo7.cible=0;
        } else{    
        Servo7.angle--;
        Servo7.servo.write(Servo7.angle);
        led=true;
      }
  }
  if(Servo7.cible==2){
      if(Servo7.angle==Servo7.borne2){
        Servo7.servo.detach();
        Servo7.cible=0;
        }else{
        Servo7.angle++;
        Servo7.servo.write(Servo7.angle);
        led=true;
      }
  }
#endif

#if servo8!=0
  if(Servo8.cible==1){
      if(Servo8.angle==Servo8.borne1){
        Servo8.servo.detach();
        Servo8.cible=0;
        } else{    
        Servo8.angle--;
        Servo8.servo.write(Servo8.angle);
        led=true;
      }
  }
  if(Servo8.cible==2){
      if(Servo8.angle==Servo8.borne2){
        Servo8.servo.detach();
        Servo8.cible=0;
        }else{
        Servo8.angle++;
        Servo8.servo.write(Servo8.angle);
        led=true;
      }
  }
#endif

// on allume ou éteind D9
  if(led==true) PORTB |=  (1<<1); else PORTB &= ~(1<<1) ;
  }// fin du millis
}
